@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.210.2.2 2006/03/09 15:27:26 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.214 2005/11/24 21:37:47 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -34,7 +34,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <sched.h>
 /* X11 */
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -454,6 +453,7 @@ bacon_video_widget_finalize (GObject *object)
 
 	if (bvw->priv->xine != NULL) {
 		xine_exit (bvw->priv->xine);
+		bvw->priv->xine = NULL;
 	}
 	if (bvw->priv->cursor != NULL) {
 		gdk_cursor_unref (bvw->priv->cursor);
@@ -1421,43 +1421,6 @@ xine_event (void *user_data, const xine_event_t *event)
 	}
 }
 
-static int
-bacon_video_widget_sort_queue (gconstpointer a, gconstpointer b, gpointer data)
-{
-	signal_data *one, *two;
-
-	one = (signal_data *) a;
-	two = (signal_data *) b;
-
-	if (one->signal == two->signal)
-		return 0;
-	if (one->signal == ERROR_ASYNC || one->signal == MESSAGE_ASYNC)
-		return 1;
-	return -1;
-}
-
-#if (!(GLIB_CHECK_VERSION(2,9,1)))
-static void
-bacon_video_widget_queue_sort (GAsyncQueue *queue)
-{
-	GList *list = NULL, *l;
-	signal_data *data;
-
-	while ((data = g_async_queue_try_pop (queue)) != NULL) {
-		list = g_list_insert_sorted (list, data, (GCompareFunc) bacon_video_widget_sort_queue);
-	}
-
-	if (list == NULL)
-		return;
-
-	for (l = list; l != NULL; l = l->next) {
-		g_async_queue_push (queue, l->data);
-	}
-	g_list_free (list);
-}
-
-#endif /* ! GLIB_CHECK_VERSION 2.9.1 */
-
 static void
 xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 {
@@ -1466,21 +1429,12 @@ xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 
 	save_data = NULL;
 
-	sched_yield ();
-
-	/* Sort the queue with the errors first */
-#if GLIB_CHECK_VERSION(2,9,1)
-	g_async_queue_sort (bvw->priv->queue, bacon_video_widget_sort_queue, bvw);
-#else
-	bacon_video_widget_queue_sort (bvw->priv->queue);
-#endif
-
 	/* Steal messages from the async queue, if there's an error,
 	 * to use as the error message rather than the crappy errors from
 	 * xine_open() */
 	while ((data = g_async_queue_try_pop (bvw->priv->queue)) != NULL)
 	{
-		if (data->signal == ERROR_ASYNC || data->signal == MESSAGE_ASYNC)
+		if (data->signal == ERROR_ASYNC)
 		{
 			if (save_data != NULL)
 			{
@@ -1489,8 +1443,8 @@ xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 			}
 			save_data = data;
 		} else {
-			g_async_queue_push (bvw->priv->queue, data);
-			break;
+			g_free (data->msg);
+			g_free (data);
 		}
 	}
 
@@ -1600,6 +1554,9 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 	xine_config_save (bvw->priv->xine, configfile);
 	g_free (configfile);
 
+	if (bvw->priv->vis != NULL)
+		xine_post_dispose (bvw->priv->xine, bvw->priv->vis);
+
 	/* Kill the drivers */
 	if (bvw->priv->vo_driver != NULL)
 		xine_close_video_driver (bvw->priv->xine,
@@ -1608,9 +1565,6 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 		xine_close_audio_driver (bvw->priv->xine,
 				bvw->priv->ao_driver);
 
-	if (bvw->priv->vis != NULL)
-		xine_post_dispose (bvw->priv->xine, bvw->priv->vis);
-
 	/* stop event thread */
 	xine_exit (bvw->priv->xine);
 	bvw->priv->xine = NULL;
@@ -1700,6 +1654,7 @@ bacon_video_widget_new (int width, int height,
 					bvw->priv->ao_driver);
 		}
 		xine_exit (bvw->priv->xine);
+		bvw->priv->xine = NULL;
 
 		/* get rid of all our crappety crap */
 		g_source_remove (bvw->priv->tick_id);
@@ -2198,6 +2153,12 @@ bacon_video_widget_play (BaconVideoWidget *bvw, GError **gerror)
 	return TRUE;
 }
 
+gboolean
+bacon_video_widget_can_direct_seek (BaconVideoWidget *bvw)
+{
+	return FALSE;
+}
+
 gboolean bacon_video_widget_seek (BaconVideoWidget *bvw, float position,
 		GError **gerror)
 {
@@ -3354,10 +3315,6 @@ bacon_video_widget_get_metadata_string (BaconVideoWidget *bvw, BaconVideoWidgetM
 		string = xine_get_meta_info (bvw->priv->stream,
 				XINE_META_INFO_AUDIOCODEC);
 		break;
-	case BVW_INFO_CDINDEX:
-		string = xine_get_meta_info (bvw->priv->stream,
-				XINE_META_INFO_CDINDEX_DISCID);
-		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -3500,7 +3457,6 @@ bacon_video_widget_get_metadata (BaconVideoWidget *bvw,
 	case BVW_INFO_YEAR:
 	case BVW_INFO_VIDEO_CODEC:
 	case BVW_INFO_AUDIO_CODEC:
-	case BVW_INFO_CDINDEX:
 		bacon_video_widget_get_metadata_string (bvw, type, value);
 		break;
 	case BVW_INFO_DURATION:
@@ -3602,7 +3558,7 @@ GList
 	num_channels = xine_get_stream_info
 		(bvw->priv->stream, XINE_STREAM_INFO_MAX_SPU_CHANNEL);
 
-	if (num_channels < 1)
+	if (num_channels < 2)
 		return NULL;
 
 	for(i = 0; i < num_channels; i++)
