@@ -1,8 +1,8 @@
 /* 
    arch-tag: Implementation of Rhythmbox playlist parser
 
-   Copyright (C) 2002, 2003, 2004 Bastien Nocera
-   Copyright (C) 2003,2004 Colin Walters <walters@rhythmbox.org>
+   Copyright (C) 2002, 2003, 2004, 2005 Bastien Nocera
+   Copyright (C) 2003, 2004 Colin Walters <walters@rhythmbox.org>
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -196,6 +196,16 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data)
 	return mimetype;
 }
 
+static char*
+totem_pl_parser_set_method (const char *url, const char *method)
+{
+	char *start;
+
+	g_return_val_if_fail (strstr (url, "://"), NULL);
+	start = strstr (url, "://");
+	return g_strdup_printf ("%s%s", method, start);
+}
+
 static char*
 totem_pl_parser_base_url (const char *url)
 {
@@ -734,107 +744,6 @@ totem_pl_parser_add_ram (TotemPlParser *parser, const char *url, gpointer data)
 	return retval;
 }
 
-static const char *
-totem_pl_parser_get_extinfo_title (gboolean extinfo, char **lines, int i)
-{
-	const char *retval;
-
-	if (extinfo == FALSE || lines == NULL)
-		return NULL;
-
-	if (i == 0)
-		return NULL;
-
-	retval = strstr (lines[i-1], "#EXTINF:");
-	retval = strstr (retval, ",");
-	if (retval == NULL || retval[0] == '\0')
-		return NULL;
-
-	retval++;
-
-	return retval;
-}
-
-static TotemPlParserResult
-totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
-{
-	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
-	char *contents, **lines;
-	int size, i;
-	const char *split_char;
-	gboolean extinfo;
-
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
-		return FALSE;
-
-	/* is TRUE if there's an EXTINF on the previous line */
-	extinfo = FALSE;
-
-	/* figure out whether we're a unix m3u or dos m3u */
-	if (strstr(contents,"\x0d") == NULL)
-		split_char = "\n";
-	else
-		split_char = "\x0d\n";
-
-	lines = g_strsplit (contents, split_char, 0);
-	g_free (contents);
-
-	for (i = 0; lines[i] != NULL; i++) {
-		if (lines[i][0] == '\0')
-			continue;
-
-		retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
-
-		/* Ignore comments, but mark it if we have extra info */
-		if (lines[i][0] == '#') {
-			if (strstr (lines[i], "#EXTINF") != NULL)
-				extinfo = TRUE;
-			continue;
-		}
-
-		/* Either it's a URI, or it has a proper path ... */
-		if (strstr(lines[i], "://") != NULL
-				|| lines[i][0] == G_DIR_SEPARATOR) {
-			totem_pl_parser_add_one_url (parser, lines[i],
-					totem_pl_parser_get_extinfo_title (extinfo, lines, i));
-			extinfo = FALSE;
-		} else if (lines[i][0] == '\\' && lines[i][1] == '\\') {
-			/* ... Or it's in the windows smb form
-			 * (\\machine\share\filename), Note drive names
-			 * (C:\ D:\ etc) are unhandled (unknown base for
-			 * drive letters) */
-		        char *tmpurl;
-
-			lines[i] = g_strdelimit (lines[i], "\\", '/');
-			tmpurl = g_strjoin (NULL, "smb:", lines[i], NULL);
-
-			totem_pl_parser_add_one_url (parser, lines[i],
-					totem_pl_parser_get_extinfo_title (extinfo, lines, i));
-			extinfo = FALSE;
-
-			g_free (tmpurl);
-		} else {
-			/* Try with a base */
-			char *fullpath, *base, sep;
-
-			base = totem_pl_parser_base_url (url);
-			sep = (split_char[0] == '\n' ? '/' : '\\');
-			if (sep == '\\')
-				lines[i] = g_strdelimit (lines[i], "\\", '/');
-			fullpath = g_strdup_printf ("%s/%s", base, lines[i]);
-			totem_pl_parser_add_one_url (parser, fullpath,
-					totem_pl_parser_get_extinfo_title (extinfo, lines, i));
-			g_free (fullpath);
-			g_free (base);
-			extinfo = FALSE;
-		}
-	}
-
-	g_strfreev (lines);
-
-	return retval;
-}
-
 static TotemPlParserResult
 totem_pl_parser_add_asf_reference_parser (TotemPlParser *parser,
 		const char *url, gpointer data)
@@ -908,23 +817,14 @@ totem_pl_parser_add_asf_parser (TotemPlParser *parser,
 }
 
 static TotemPlParserResult
-totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
+totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url, const char *contents)
 {
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
-	char *contents, **lines;
-	int size, i, num_entries;
+	char **lines;
+	int i, num_entries;
 	char *split_char, *playlist_title;
 	gboolean dos_mode = FALSE;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
-		return TOTEM_PL_PARSER_RESULT_ERROR;
-
-	if (size == 0)
-	{
-		g_free (contents);
-		return TOTEM_PL_PARSER_RESULT_SUCCESS;
-	}
-
 	/* figure out whether we're a unix pls or dos pls */
 	if (strstr(contents,"\x0d") == NULL) {
 		split_char = "\n";
@@ -933,7 +833,6 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 		dos_mode = TRUE;
 	}
 	lines = g_strsplit (contents, split_char, 0);
-	g_free (contents);
 
 	/* [playlist] */
 	i = 0;
@@ -1028,13 +927,144 @@ bail:
 	return retval;
 }
 
+static TotemPlParserResult
+totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
+{
+	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	char *contents;
+	int size;
+
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+		return TOTEM_PL_PARSER_RESULT_ERROR;
+
+	if (size == 0) {
+		g_free (contents);
+		return TOTEM_PL_PARSER_RESULT_SUCCESS;
+	}
+
+	retval = totem_pl_parser_add_pls_with_contents (parser, url, contents);
+	g_free (contents);
+
+	return retval;
+}
+
+static const char *
+totem_pl_parser_get_extinfo_title (gboolean extinfo, char **lines, int i)
+{
+	const char *retval;
+
+	if (extinfo == FALSE || lines == NULL)
+		return NULL;
+
+	if (i == 0)
+		return NULL;
+
+	retval = strstr (lines[i-1], "#EXTINF:");
+	retval = strstr (retval, ",");
+	if (retval == NULL || retval[0] == '\0')
+		return NULL;
+
+	retval++;
+
+	return retval;
+}
+
+static TotemPlParserResult
+totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
+{
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	char *contents, **lines;
+	int size, i;
+	const char *split_char;
+	gboolean extinfo;
+
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+		return TOTEM_PL_PARSER_RESULT_ERROR;
+
+	/* .pls files with a .m3u extension, the nasties */
+	if (g_str_has_prefix (contents, "[playlist]") != FALSE
+			|| g_str_has_prefix (contents, "[Playlist]") != FALSE
+			|| g_str_has_prefix (contents, "[PLAYLIST]") != FALSE) {
+		retval = totem_pl_parser_add_pls_with_contents (parser, url, contents);
+		g_free (contents);
+		return retval;
+	}
+
+	/* is TRUE if there's an EXTINF on the previous line */
+	extinfo = FALSE;
+
+	/* figure out whether we're a unix m3u or dos m3u */
+	if (strstr(contents,"\x0d") == NULL)
+		split_char = "\n";
+	else
+		split_char = "\x0d\n";
+
+	lines = g_strsplit (contents, split_char, 0);
+	g_free (contents);
+
+	for (i = 0; lines[i] != NULL; i++) {
+		if (lines[i][0] == '\0')
+			continue;
+
+		retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
+
+		/* Ignore comments, but mark it if we have extra info */
+		if (lines[i][0] == '#') {
+			if (strstr (lines[i], "#EXTINF") != NULL)
+				extinfo = TRUE;
+			continue;
+		}
+
+		/* Either it's a URI, or it has a proper path ... */
+		if (strstr(lines[i], "://") != NULL
+				|| lines[i][0] == G_DIR_SEPARATOR) {
+			totem_pl_parser_add_one_url (parser, lines[i],
+					totem_pl_parser_get_extinfo_title (extinfo, lines, i));
+			extinfo = FALSE;
+		} else if (lines[i][0] == '\\' && lines[i][1] == '\\') {
+			/* ... Or it's in the windows smb form
+			 * (\\machine\share\filename), Note drive names
+			 * (C:\ D:\ etc) are unhandled (unknown base for
+			 * drive letters) */
+		        char *tmpurl;
+
+			lines[i] = g_strdelimit (lines[i], "\\", '/');
+			tmpurl = g_strjoin (NULL, "smb:", lines[i], NULL);
+
+			totem_pl_parser_add_one_url (parser, lines[i],
+					totem_pl_parser_get_extinfo_title (extinfo, lines, i));
+			extinfo = FALSE;
+
+			g_free (tmpurl);
+		} else {
+			/* Try with a base */
+			char *fullpath, *base, sep;
+
+			base = totem_pl_parser_base_url (url);
+			sep = (split_char[0] == '\n' ? '/' : '\\');
+			if (sep == '\\')
+				lines[i] = g_strdelimit (lines[i], "\\", '/');
+			fullpath = g_strdup_printf ("%s/%s", base, lines[i]);
+			totem_pl_parser_add_one_url (parser, fullpath,
+					totem_pl_parser_get_extinfo_title (extinfo, lines, i));
+			g_free (fullpath);
+			g_free (base);
+			extinfo = FALSE;
+		}
+	}
+
+	g_strfreev (lines);
+
+	return retval;
+}
+
 static gboolean
 parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		xmlNodePtr parent, const char *pl_title)
 {
 	xmlNodePtr node;
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 	guchar *title, *url;
-	gboolean retval = FALSE;
 	char *fullpath = NULL;
 
 	title = NULL;
@@ -1059,7 +1089,7 @@ parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 
 	if (url == NULL) {
 		g_free (title);
-		return FALSE;
+		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
 
 	if (strstr ((char *)url, "://") == NULL && url[0] != '/') {
@@ -1068,15 +1098,24 @@ parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		fullpath = g_strdup ((char *)url);
 	}
 
+	g_free (url);
+
+	if (g_str_has_prefix (fullpath, "mms:") != FALSE) {
+		char *tmp;
+		tmp = totem_pl_parser_set_method (fullpath, "mmsh");
+		g_free (fullpath);
+		fullpath = tmp;
+	}
+
 	/* .asx files can contain references to other .asx files */
-	if (totem_pl_parser_parse_internal (parser, fullpath) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+	retval = totem_pl_parser_parse_internal (parser, fullpath);
+	if (retval != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 		totem_pl_parser_add_one_url (parser, fullpath,
 				(char *)title ? (char *)title : pl_title);
 	}
 
 	g_free (fullpath);
 	g_free (title);
-	g_free (url);
 
 	return retval;
 }
@@ -1087,7 +1126,7 @@ parse_asx_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 {
 	guchar *title = NULL;
 	xmlNodePtr node;
-	gboolean retval = FALSE;
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_ERROR;
 
 	for (node = parent->children; node != NULL; node = node->next) {
 		if (node->name == NULL)
@@ -1100,18 +1139,18 @@ parse_asx_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		if (g_ascii_strcasecmp ((char *)node->name, "entry") == 0) {
 			/* Whee found an entry here, find the REF and TITLE */
 			if (parse_asx_entry (parser, base, doc, node, (char *)title) != FALSE)
-				retval = TRUE;
+				retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 		}
 		if (g_ascii_strcasecmp ((char *)node->name, "entryref") == 0) {
 			/* Found an entryref, give the parent instead of the
 			 * children to the parser */
 			if (parse_asx_entry (parser, base, doc, parent, (char *)title) != FALSE)
-				retval = TRUE;
+				retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 		}
 		if (g_ascii_strcasecmp ((char *)node->name, "repeat") == 0) {
 			/* Repeat at the top-level */
 			if (parse_asx_entries (parser, base, doc, node) != FALSE)
-				retval = TRUE;
+				retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 		}
 	}
 
@@ -1193,7 +1232,7 @@ parse_smil_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 {
 	xmlNodePtr node;
 	guchar *title, *url;
-	gboolean retval = FALSE;
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_ERROR;
 
 	title = NULL;
 	url = NULL;
@@ -1211,7 +1250,7 @@ parse_smil_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 			if (url != NULL) {
 				if (parse_smil_video_entry (parser,
 						base, (char *)url, (char *)title) != FALSE)
-					retval = TRUE;
+					retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 			}
 
 			g_free (title);
@@ -1219,7 +1258,7 @@ parse_smil_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		} else {
 			if (parse_smil_entry (parser,
 						base, doc, node) != FALSE)
-				retval = TRUE;
+				retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 		}
 	}
 
@@ -1231,7 +1270,7 @@ parse_smil_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		xmlNodePtr parent)
 {
 	xmlNodePtr node;
-	gboolean retval = FALSE;
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_ERROR;
 
 	for (node = parent->children; node != NULL; node = node->next) {
 		if (node->name == NULL)
@@ -1240,7 +1279,7 @@ parse_smil_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		if (g_ascii_strcasecmp ((char *)node->name, "body") == 0) {
 			if (parse_smil_entry (parser, base,
 						doc, node) != FALSE)
-				retval = TRUE;
+				retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 		}
 
 	}
@@ -1255,7 +1294,7 @@ totem_pl_parser_add_smil (TotemPlParser *parser, const char *url, gpointer data)
 	xmlNodePtr node;
 	char *contents = NULL, *base;
 	int size;
-	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
 	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
@@ -1293,7 +1332,8 @@ totem_pl_parser_add_asf (TotemPlParser *parser, const char *url, gpointer data)
 	if (data != NULL &&
 		(g_str_has_prefix (data, "[Reference]") == FALSE
 		 && g_ascii_strncasecmp (data, "<ASX", strlen ("<ASX")) != 0
-		 && strstr (data, "<ASX") == NULL)
+		 && strstr (data, "<ASX") == NULL
+		 && strstr (data, "<asx") == NULL)
 		 && g_str_has_prefix (data, "ASF ") == FALSE) {
 		totem_pl_parser_add_one_url (parser, url, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
@@ -1357,6 +1397,49 @@ totem_pl_parser_dir_compare (GnomeVFSFileInfo *a, GnomeVFSFileInfo *b)
 	}
 }
 
+static TotemPlParserResult
+totem_pl_parser_add_iso (TotemPlParser *parser, const char *url,
+		gpointer data)
+{
+	GnomeVFSFileInfo *info;
+	char *item;
+
+	/* This is a hack, it could be a VCD or DVD */
+	if (g_str_has_prefix (url, "file://") == FALSE)
+		return TOTEM_PL_PARSER_RESULT_IGNORED;
+	info = gnome_vfs_file_info_new ();
+	if (gnome_vfs_get_file_info (url, info, GNOME_VFS_FILE_INFO_FOLLOW_LINKS) != GNOME_VFS_OK) {
+		gnome_vfs_file_info_unref (info);
+		return TOTEM_PL_PARSER_RESULT_IGNORED;
+	}
+
+	/* Less than 700 megs, and it's a VCD */
+	if (info->size < 700 * 1024 * 1024) {
+		item = totem_cd_mrl_from_type ("vcd", url);
+	} else {
+		item = totem_cd_mrl_from_type ("dvd", url);
+	}
+
+	gnome_vfs_file_info_unref (info);
+	totem_pl_parser_add_one_url (parser, item, NULL);
+	g_free (item);
+
+	return TOTEM_PL_PARSER_RESULT_SUCCESS;
+}
+
+static TotemPlParserResult
+totem_pl_parser_add_cue (TotemPlParser *parser, const char *url,
+		gpointer data)
+{
+	char *vcdurl;
+
+	vcdurl = totem_cd_mrl_from_type ("vcd", url);
+	totem_pl_parser_add_one_url (parser, vcdurl, NULL);
+	g_free (vcdurl);
+
+	return TOTEM_PL_PARSER_RESULT_SUCCESS;
+}
+
 static TotemPlParserResult
 totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 			   gpointer data)
@@ -1426,6 +1509,8 @@ static PlaylistTypes special_types[] = {
 	{ "x-directory/normal", totem_pl_parser_add_directory },
 	{ "video/x-ms-wvx", totem_pl_parser_add_asx },
 	{ "audio/x-ms-wax", totem_pl_parser_add_asx },
+	{ "application/x-cd-image", totem_pl_parser_add_iso },
+	{ "application/x-cue", totem_pl_parser_add_cue },
 };
 
 static PlaylistTypes ignore_types[] = {
@@ -1433,6 +1518,7 @@ static PlaylistTypes ignore_types[] = {
 	{ "text/plain", NULL },
 	{ "application/x-rar", NULL },
 	{ "application/zip", NULL },
+	{ "application/x-trash", NULL },
 };
 
 /* These ones are "dual" types, might be a video, might be a parser */
@@ -1521,15 +1607,16 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	const char *mimetype;
 	guint i;
 	gpointer data = NULL;
-	gboolean ret = FALSE;
+	TotemPlParserResult ret = FALSE;
 	char *super;
 
 	if (parser->priv->recurse_level > RECURSE_LEVEL_MAX)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	mimetype = gnome_vfs_get_mime_type (url);
-	if (!mimetype)
+	mimetype = gnome_vfs_mime_type_from_name (url);
+	if (mimetype == NULL || strcmp (GNOME_VFS_MIME_TYPE_UNKNOWN, mimetype) == 0) {
 		mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data);
+	}
 
 	if (mimetype == NULL)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
@@ -1581,12 +1668,12 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 
 	parser->priv->recurse_level--;
 
-	if (ret == FALSE && parser->priv->fallback) {
+	if (ret != TOTEM_PL_PARSER_RESULT_SUCCESS && parser->priv->fallback) {
 		totem_pl_parser_add_one_url (parser, url, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
-	if (ret)
+	if (ret == TOTEM_PL_PARSER_RESULT_SUCCESS)
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	else
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
