@@ -41,26 +41,40 @@
 #include "npapi.h"
 #include "npupp.h"
 
-#include <nsCOMPtr.h>
 #include <nsIDOMWindow.h>
 #include <nsIURI.h>
 #include <nsEmbedString.h>
 #include <nsIInterfaceRequestorUtils.h>
 #include <docshell/nsIWebNavigation.h>
 
-#define DEBUG(x) printf(x "\n")
-//#define DEBUG(x)
+/* define TOTEM_DEBUG for more debug spew */
+#include "debug.h"
 
 static NPNetscapeFuncs mozilla_functions;
+static char *mime_list = NULL;
+
+static void
+cb_update_name (DBusGProxy *proxy, char *svc, char *old_owner,
+		char *new_owner, TotemPlugin *plugin)
+{
+	D("Received notification for %s\n", svc);
+	if (strcmp (svc, plugin->wait_for_svc) == 0) {
+		plugin->got_svc = TRUE;
+	}
+}
 
 /* You don't update, you die! */
 #define MAX_ARGV_LEN 14
 
-static void totem_plugin_fork (TotemPlugin *plugin)
+static gboolean
+totem_plugin_fork (TotemPlugin *plugin)
 {
+	GTimeVal then, now;
+	char *svcname;
 	char **argv;
 	int argc = 0;
 	GError *err = NULL;
+	totemMozillaObject *iface = plugin->iface;
 
 	argv = (char **)g_new0 (char *, MAX_ARGV_LEN);
 
@@ -101,6 +115,7 @@ static void totem_plugin_fork (TotemPlugin *plugin)
 	argv[argc++] = g_strdup ("fd://0");
 	argv[argc] = NULL;
 
+#ifdef TOTEM_DEBUG
 	{
 		int i;
 		g_print ("Launching: ");
@@ -109,32 +124,79 @@ static void totem_plugin_fork (TotemPlugin *plugin)
 		}
 		g_print ("\n");
 	}
+#endif
 
 	if (g_spawn_async_with_pipes (NULL, argv, NULL,
 				G_SPAWN_DO_NOT_REAP_CHILD,
 				NULL, NULL, &plugin->player_pid,
 				&plugin->send_fd, NULL, NULL, &err) == FALSE)
 	{
-		DEBUG("Spawn failed");
+		D("Spawn failed");
 
-		if(err)
+		if (err)
 		{
 			fprintf(stderr, "%s\n", err->message);
 			g_error_free(err);
 		}
+
+		g_strfreev (argv);
+		return FALSE;
 	}
 
 	g_strfreev (argv);
-}
 
-static void
-cb_data (const gchar * msg, gpointer user_data)
-{
-	TotemPlugin *plugin = (TotemPlugin *) user_data;
+	/* now wait until startup is complete */
+	plugin->got_svc = FALSE;
+	plugin->wait_for_svc =
+		g_strdup_printf ("org.totem_%d.MozillaPluginService",
+				 plugin->player_pid);
+	D("waiting for signal %s", plugin->wait_for_svc);
+	dbus_g_proxy_add_signal (plugin->proxy, "NameOwnerChanged",
+				 G_TYPE_STRING, G_TYPE_STRING,
+				 G_TYPE_STRING, G_TYPE_INVALID);
+	dbus_g_proxy_connect_signal (plugin->proxy, "NameOwnerChanged",
+				     G_CALLBACK (cb_update_name),
+				     plugin, NULL);
+	g_get_current_time (&then);
+	g_time_val_add (&then, G_USEC_PER_SEC * 5);
+	NS_ADDREF (iface);
+	do {
+		g_main_context_iteration (NULL, TRUE);
+		g_get_current_time (&now);
+	} while (iface->tm != NULL && !plugin->got_svc &&
+		 (now.tv_sec <= then.tv_sec));
+	if (!iface->tm) {
+		/* we were destroyed in one of the iterations of the
+		 * mainloop, get out ASAP */
+		D("We no longer exist");
+		NS_RELEASE (iface);
+		return FALSE;
+	}
+	NS_RELEASE (iface);
+	dbus_g_proxy_disconnect_signal (plugin->proxy, "NameOwnerChanged",
+					G_CALLBACK (cb_update_name), plugin);
+	if (!plugin->got_svc) {
+		fprintf (stderr, "Failed to receive DBUS interface response\n");
+		g_free (plugin->wait_for_svc);
 
-	g_free (plugin->last_msg);
-	plugin->last_msg = g_strdup (msg);
-	g_print ("Read msg '%s'\n", msg);
+		if (plugin->player_pid) {
+			kill (plugin->player_pid, SIGKILL);
+			waitpid (plugin->player_pid, NULL, 0);
+			plugin->player_pid = 0;
+		}
+		return FALSE;
+	}
+	g_object_unref (plugin->proxy);
+
+	/* now get the proxy for the player functions */
+	plugin->proxy =
+		dbus_g_proxy_new_for_name (plugin->conn, plugin->wait_for_svc,
+					   "/TotemEmbedded",
+					   "org.totem.MozillaPluginInterface");
+	g_free (plugin->wait_for_svc);
+	D("Done forking, new proxy=%p", plugin->proxy);
+
+	return TRUE;
 }
 
 static char *
@@ -157,9 +219,10 @@ static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
 		NPSavedData *saved)
 {
 	TotemPlugin *plugin;
+	GError *e = NULL;
 	int i;
 
-	DEBUG("totem_plugin_new_instance");
+	D("totem_plugin_new_instance");
 
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
@@ -176,14 +239,24 @@ static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
 		mozilla_functions.memfree (plugin);
 		return NPERR_OUT_OF_MEMORY_ERROR;
 	}
-	plugin->conn = bacon_message_connection_new ("totem-mozilla");
-	if (!plugin->conn) {
-		delete plugin->iface;
+	NS_ADDREF (plugin->iface);
+	if (!(plugin->conn = dbus_g_bus_get (DBUS_BUS_SESSION, &e))) {
+		printf ("Failed to open DBUS session: %s\n", e->message);
+		g_error_free (e);
+		NS_RELEASE (plugin->iface);
+		mozilla_functions.memfree (plugin);
+		return NPERR_OUT_OF_MEMORY_ERROR;
+	} else if (!(plugin->proxy = dbus_g_proxy_new_for_name (plugin->conn,
+					"org.freedesktop.DBus",
+					"/org/freedesktop/DBus",
+					"org.freedesktop.DBus"))) {
+		printf ("Failed to open DBUS proxy: %s\n", e->message);
+		g_error_free (e);
+		g_object_unref (G_OBJECT (plugin->conn));
+		NS_RELEASE (plugin->iface);
 		mozilla_functions.memfree (plugin);
 		return NPERR_OUT_OF_MEMORY_ERROR;
 	}
-	bacon_message_connection_set_callback (plugin->conn, cb_data, plugin);
-	//NS_ADDREF (plugin->iface);
 
 	/* mode is NP_EMBED, NP_FULL, or NP_BACKGROUND (see npapi.h) */
 	printf("mode %d\n",mode);
@@ -192,14 +265,20 @@ static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
 	plugin->send_fd = -1;
 
 	/* to resolve relative URLs */
-	nsCOMPtr<nsIDOMWindow> domWin;
-	mozilla_functions.getvalue (instance, NPNVDOMWindow, 
-				    NS_STATIC_CAST (nsIDOMWindow **, getter_AddRefs (domWin)));
-	nsCOMPtr<nsIWebNavigation> webNav (do_GetInterface (domWin));
+	nsIDOMWindow *domWin = nsnull;
+	mozilla_functions.getvalue (instance, NPNVDOMWindow,
+				    NS_REINTERPRET_CAST (void**, &domWin));
+	nsIWebNavigation *webNav = nsnull;
+	if (domWin) {
+		domWin->QueryInterface (NS_GET_IID (nsIWebNavigation),
+					NS_REINTERPRET_CAST (void**, &webNav));
+		NS_RELEASE (domWin);
+	}
 
-	nsCOMPtr<nsIURI> docURI;
+	nsIURI *docURI = nsnull;
 	if (webNav) {
-		webNav->GetCurrentURI (getter_AddRefs (docURI));
+		webNav->GetCurrentURI (&docURI);
+		NS_RELEASE (webNav);
 	}
 
 	for (i=0; i<argc; i++) {
@@ -241,6 +320,8 @@ static NPError totem_plugin_new_instance (NPMIMEType mime_type, NPP instance,
 		}
 	}
 
+	NS_IF_RELEASE (docURI);	
+
 	return NPERR_NO_ERROR;
 }
 
@@ -248,7 +329,7 @@ static NPError totem_plugin_destroy_instance (NPP instance, NPSavedData **save)
 {
 	TotemPlugin * plugin;
 
-	DEBUG("plugin_destroy");
+	D("plugin_destroy");
 
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
@@ -257,8 +338,11 @@ static NPError totem_plugin_destroy_instance (NPP instance, NPSavedData **save)
 	if (plugin == NULL)
 		return NPERR_NO_ERROR;
 
+	if (!plugin || !plugin->iface || !plugin->iface->tm)
+		return NPERR_INVALID_INSTANCE_ERROR;
+
+	plugin->iface->Rewind ();
 	plugin->iface->invalidatePlugin ();
-	NS_RELEASE (plugin->iface);
 
 	if (plugin->send_fd >= 0)
 		close(plugin->send_fd);
@@ -268,8 +352,10 @@ static NPError totem_plugin_destroy_instance (NPP instance, NPSavedData **save)
 		waitpid (plugin->player_pid, NULL, 0);
 	}
 
-	bacon_message_connection_free (plugin->conn);
-	g_free (plugin->last_msg);
+	NS_RELEASE (plugin->iface);
+
+	g_object_unref (G_OBJECT (plugin->proxy));
+	//g_object_unref (G_OBJECT (plugin->conn));
 	mozilla_functions.memfree (instance->pdata);
 	instance->pdata = NULL;
 
@@ -280,7 +366,7 @@ static NPError totem_plugin_set_window (NPP instance, NPWindow* window)
 {
 	TotemPlugin *plugin;
 
-	DEBUG("plugin_set_window");
+	D("plugin_set_window");
 
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
@@ -290,29 +376,29 @@ static NPError totem_plugin_set_window (NPP instance, NPWindow* window)
 		return NPERR_INVALID_INSTANCE_ERROR;
 
 	if (plugin->window) {
-		DEBUG ("existing window");
+		D ("existing window");
 		if (plugin->window == (Window) window->window) {
-			DEBUG("resize");
+			D("resize");
 			/* Resize event */
 			/* Not currently handled */
 		} else {
-			DEBUG("change");
+			D("change");
 			printf ("ack.  window changed!\n");
 		}
 	} else {
 		gchar *msg;
 
-		DEBUG("about to fork");
+		D("about to fork");
 
 		plugin->window = (Window) window->window;
-		totem_plugin_fork (plugin);
-                msg = plugin->iface->wait ();
+		if (!totem_plugin_fork (plugin))
+			return NPERR_GENERIC_ERROR;
 
 		if (plugin->send_fd > 0)
 			fcntl(plugin->send_fd, F_SETFL, O_NONBLOCK);
 	}
 
-	DEBUG("leaving plugin_set_window");
+	D("leaving plugin_set_window");
 
 	return NPERR_NO_ERROR;
 }
@@ -320,7 +406,7 @@ static NPError totem_plugin_set_window (NPP instance, NPWindow* window)
 static NPError totem_plugin_new_stream (NPP instance, NPMIMEType type,
 		NPStream* stream_ptr, NPBool seekable, uint16* stype)
 {
-	DEBUG("plugin_new_stream");
+	D("plugin_new_stream");
 
 	if (instance == NULL)
 		return NPERR_INVALID_INSTANCE_ERROR;
@@ -333,10 +419,10 @@ static NPError totem_plugin_destroy_stream (NPP instance, NPStream* stream,
 {
 	TotemPlugin *plugin;
 
-	DEBUG("plugin_destroy_stream");
+	D("plugin_destroy_stream");
 
 	if (instance == NULL) {
-		DEBUG("totem_plugin_destroy_stream instance is NULL");
+		D("totem_plugin_destroy_stream instance is NULL");
 		return NPERR_NO_ERROR;
 	}
 
@@ -352,10 +438,10 @@ static int32 totem_plugin_write_ready (NPP instance, NPStream *stream)
 {
 	TotemPlugin *plugin;
 
-	DEBUG("plugin_write_ready");
+	D("plugin_write_ready");
 
 	if (instance == NULL) {
-		DEBUG("plugin_write_ready instance is NULL");
+		D("plugin_write_ready instance is NULL");
 		return 0;
 	}
 
@@ -373,27 +459,26 @@ static int32 totem_plugin_write (NPP instance, NPStream *stream, int32 offset,
 	TotemPlugin *plugin;
 	int ret;
 
-	DEBUG("plugin_write");
+	D("plugin_write");
 
 	if (instance == NULL)
-		return 0;
+		return -1;
 
 	plugin = (TotemPlugin *) instance->pdata;
 
 	if (plugin == NULL)
-		return 0;
+		return -1;
 
 	if (!plugin->player_pid)
-		return 0;
+		return -1;
 
 	if (plugin->send_fd < 0)
-		return 0;
+		return -1;
 
 //	g_message ("write %d %p %d", plugin->send_fd, buffer, len);
 	ret = write (plugin->send_fd, buffer, len);
 	if (ret < 0) {
 		g_message ("ret %d", ret);
-		ret = 0;
 	}
 
 	return ret;
@@ -404,7 +489,7 @@ static void totem_plugin_stream_as_file (NPP instance, NPStream *stream,
 {
 	TotemPlugin *plugin;
 
-	DEBUG("plugin_stream_as_file");
+	D("plugin_stream_as_file");
 
 	if (instance == NULL)
 		return;
@@ -420,7 +505,13 @@ static void
 totem_plugin_url_notify (NPP instance, const char* url,
 		NPReason reason, void* notifyData)
 {
-	DEBUG("plugin_url_notify");
+	D("plugin_url_notify");
+}
+
+static char *
+totem_plugin_get_description (void)
+{
+	return "The <a href=\"http://www.gnome.org/projects/totem/\">Totem</a> "PACKAGE_VERSION" plugin handles video and audio streams.";
 }
 
 static NPError
@@ -437,8 +528,7 @@ totem_plugin_get_value (NPP instance, NPPVariable variable,
 		*((char **)value) = "Totem Mozilla Plugin";
 		break;
 	case NPPVpluginDescriptionString:
-		*((char **)value) =
-			"The <a href=\"http://www.gnome.org/projects/totem/\">Totem</a> plugin handles video and audio streams.";
+		*((char **)value) = totem_plugin_get_description();
 		break;
 	case NPPVpluginNeedsXEmbed:
 		*((PRBool *)value) = PR_TRUE;
@@ -450,7 +540,7 @@ totem_plugin_get_value (NPP instance, NPPVariable variable,
 		if (ptr) {
 			*ptr = sIID;
 			* (nsIID **) value = ptr;
-			g_print ("Returning that we support iface\n");
+			D("Returning that we support iface");
 		} else {
 			err = NPERR_OUT_OF_MEMORY_ERROR;
 		}
@@ -461,18 +551,17 @@ totem_plugin_get_value (NPP instance, NPPVariable variable,
 			err = NPERR_GENERIC_ERROR;
 		} else {
 		        plugin = (TotemPlugin *) instance->pdata;
-			NS_ENSURE_TRUE (plugin && plugin->iface, NPERR_INVALID_INSTANCE_ERROR);
+			NS_ENSURE_TRUE (plugin && plugin->iface && plugin->iface->tm, NPERR_INVALID_INSTANCE_ERROR);
 
-			NS_ADDREF (plugin->iface);
 			plugin->iface->QueryInterface (NS_GET_IID (nsISupports),
 						       (void **) value);
 //			* (nsISupports **) value = static_cast<totemMozillaScript *>(plugin->iface);
-			g_print ("Returning instance %p\n", plugin->iface);
+			D("Returning instance %p", plugin->iface);
 		}
 		break;
 	}
 	default:
-		g_message ("unhandled variable %d", variable);
+		D("unhandled variable %d", variable);
 		err = NPERR_INVALID_PARAM;
 		break;
 	}
@@ -484,7 +573,7 @@ static NPError
 totem_plugin_set_value (NPP instance, NPNVariable variable,
 		                        void *value)
 {
-	DEBUG("plugin_set_value");
+	D("plugin_set_value");
 
 	return NPERR_NO_ERROR;
 }
@@ -495,26 +584,30 @@ NP_GetValue(void *future, NPPVariable variable, void *value)
 	return totem_plugin_get_value (NULL, variable, value);
 }
 
-#define NUM_MIME_TYPES 6
 static struct {
 	const char *mime_type;
 	const char *extensions;
 	const char *mime_alias;
 } mime_types[] = {
-	{ "video/quicktime", "mov" },
+	{ "video/quicktime", "mov", NULL },
 	{ "application/x-mplayer2", "avi, wma, wmv", "video/x-msvideo" },
-	{ "video/mpeg", "mpg, mpeg, mpe" },
+	{ "video/mpeg", "mpg, mpeg, mpe", NULL },
 	{ "video/x-ms-asf-plugin", "asf, wmv", "video/x-ms-asf" },
 	{ "video/x-ms-wmv", "wmv", "video/x-ms-asf" },
-	{ "application/ogg", "ogg" }
+	{ "application/ogg", "ogg", NULL },
+	{ "application/x-google-vlc-plugin", "", "Google VLC plugin" },
+	{ "application/x-vlc-plugin", "", "VideoLAN plugin" }
 };
+#define NUM_MIME_TYPES G_N_ELEMENTS(mime_types)
 
 char *NP_GetMIMEDescription(void)
 {
 	GString *list;
-	char *mime_list;
 	guint i;
 
+	if (mime_list != NULL)
+		return mime_list;
+
 	list = g_string_new (NULL);
 
 	for (i = 0; i < NUM_MIME_TYPES; i++) {
@@ -522,10 +615,14 @@ char *NP_GetMIMEDescription(void)
 		char *item;
 
 		desc = gnome_vfs_mime_get_description (mime_types[i].mime_type);
-		if (desc == NULL) {
+		if (desc == NULL && mime_types[i].mime_alias != NULL) {
 			desc = gnome_vfs_mime_get_description
 				(mime_types[i].mime_alias);
 		}
+		if (desc == NULL) {
+			desc = mime_types[i].mime_alias;
+		}
+
 		item = g_strdup_printf ("%s:%s:%s;", mime_types[i].mime_type,
 				mime_types[i].extensions, desc);
 		list = g_string_append (list, item);
@@ -639,6 +736,9 @@ NPError NP_Initialize (NPNetscapeFuncs * moz_funcs,
 
 NPError NP_Shutdown(void)
 {
+	g_free (mime_list);
+	mime_list = NULL;
+
 	return NPERR_NO_ERROR;
 }
 
